@@ -1,9 +1,13 @@
+//go:build scheduler.none
 // +build scheduler.none
 
 package task
 
 import "unsafe"
 
+// There is only one goroutine so the task struct can be a global.
+var mainTask Task
+
 //go:linkname runtimePanic runtime.runtimePanic
 func runtimePanic(str string)
 
@@ -12,8 +16,8 @@ func Pause() {
 }
 
 func Current() *Task {
-	runtimePanic("scheduler is disabled")
-	return nil
+	// Return a task struct, which is used for the recover builtin for example.
+	return &mainTask
 }
 
 //go:noinline
