@@ -1,9 +1,10 @@
+//go:build circuitplay_express
 // +build circuitplay_express
 
 package machine
 
 // used to reset into bootloader
-const RESET_MAGIC_VALUE = 0xf01669ef
+const resetMagicValue = 0xf01669ef
 
 // GPIO Pins
 const (
