@@ -1,9 +1,10 @@
+//go:build itsybitsy_m4
 // +build itsybitsy_m4
 
 package machine
 
 // used to reset into bootloader
-const RESET_MAGIC_VALUE = 0xf01669ef
+const resetMagicValue = 0xf01669ef
 
 // GPIO Pins
 const (
@@ -54,12 +55,24 @@ const (
 	UART2_RX_PIN = D2
 )
 
+var (
+	UART1 = &sercomUSART3
+	UART2 = &sercomUSART0
+
+	DefaultUART = UART1
+)
+
 // I2C pins
 const (
 	SDA_PIN = PA12 // SDA: SERCOM2/PAD[0]
 	SCL_PIN = PA13 // SCL: SERCOM2/PAD[1]
 )
 
+// I2C on the ItsyBitsy M4.
+var (
+	I2C0 = sercomI2CM2
+)
+
 // SPI pins
 const (
 	SPI0_SCK_PIN = PA01 // SCK: SERCOM1/PAD[1]
@@ -67,6 +80,9 @@ const (
 	SPI0_SDI_PIN = PB23 // SDI: SERCOM1/PAD[3]
 )
 
+// SPI on the ItsyBitsy M4.
+var SPI0 = sercomSPIM1
+
 // USB CDC identifiers
 const (
 	usb_STRING_PRODUCT      = "Adafruit ItsyBitsy M4"
