@@ -1,3 +1,4 @@
+//go:build nodemcu
 // +build nodemcu
 
 // Pinout for the NodeMCU dev kit.
@@ -6,15 +7,15 @@ package machine
 
 // GPIO pins on the NodeMCU board.
 const (
-	D0 Pin = 16
-	D1 Pin = 5
-	D2 Pin = 4
-	D3 Pin = 0
-	D4 Pin = 2
-	D5 Pin = 14
-	D6 Pin = 12
-	D7 Pin = 13
-	D8 Pin = 15
+	D0 = GPIO16
+	D1 = GPIO5
+	D2 = GPIO4
+	D3 = GPIO0
+	D4 = GPIO2
+	D5 = GPIO14
+	D6 = GPIO12
+	D7 = GPIO13
+	D8 = GPIO15
 )
 
 // Onboard blue LED (on the AI-Thinker module).
