@@ -1,3 +1,4 @@
+//go:build nucleol552ze
 // +build nucleol552ze
 
 package machine
@@ -8,11 +9,11 @@ import (
 )
 
 const (
-	LED         = LED_BUILTIN
-	LED_BUILTIN = LED_GREEN
 	LED_GREEN   = PC7
 	LED_BLUE    = PB7
 	LED_RED     = PA9
+	LED_BUILTIN = LED_GREEN
+	LED         = LED_BUILTIN
 )
 
 const (
@@ -31,15 +32,30 @@ const (
 var (
 	// LPUART1 is the hardware serial port connected to the onboard ST-LINK
 	// debugger to be exposed as virtual COM port over USB on Nucleo boards.
-	// Both UART0 and UART1 refer to LPUART1.
-	UART0 = UART{
-		Buffer:          NewRingBuffer(),
-		Bus:             stm32.LPUART1,
-		AltFuncSelector: UART_ALT_FN,
+	UART1  = &_UART1
+	_UART1 = UART{
+		Buffer:            NewRingBuffer(),
+		Bus:               stm32.LPUART1,
+		TxAltFuncSelector: UART_ALT_FN,
+		RxAltFuncSelector: UART_ALT_FN,
 	}
-	UART1 = &UART0
+	DefaultUART = UART1
+)
+
+const (
+	I2C0_SCL_PIN = PB8
+	I2C0_SDA_PIN = PB9
+)
+
+var (
+	// I2C1 is documented, alias to I2C0 as well
+	I2C1 = &I2C{
+		Bus:             stm32.I2C1,
+		AltFuncSelector: 4,
+	}
+	I2C0 = I2C1
 )
 
 func init() {
-	UART0.Interrupt = interrupt.New(stm32.IRQ_LPUART1, UART0.handleInterrupt)
+	UART1.Interrupt = interrupt.New(stm32.IRQ_LPUART1, _UART1.handleInterrupt)
 }
