@@ -1,12 +1,12 @@
 package runtime
 
-import "device"
-
 const GOARCH = "amd64"
 
 // The bitness of the CPU (e.g. 8, 32, 64).
 const TargetBits = 64
 
+const deferExtraRegs = 0
+
 // Align a pointer.
 // Note that some amd64 instructions (like movaps) expect 16-byte aligned
 // memory, thus the result must be 16-byte aligned.
@@ -15,5 +15,5 @@ func align(ptr uintptr) uintptr {
 }
 
 func getCurrentStackPointer() uintptr {
-	return device.AsmFull("movq %rsp, {}", nil)
+	return uintptr(stacksave())
 }
