@@ -1,3 +1,4 @@
+//go:build arm7tdmi
 // +build arm7tdmi
 
 package runtime
@@ -13,6 +14,16 @@ func putchar(c byte) {
 	// dummy, TODO
 }
 
+func getchar() byte {
+	// dummy, TODO
+	return 0
+}
+
+func buffered() int {
+	// dummy, TODO
+	return 0
+}
+
 //go:extern _sbss
 var _sbss [0]byte
 
@@ -29,6 +40,7 @@ var _sidata [0]byte
 var _edata [0]byte
 
 // Entry point for Go. Initialize all packages and call main.main().
+//
 //export main
 func main() {
 	// Initialize .data and .bss sections.
